"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const compiler_cli_1 = require("@angular/compiler-cli");
const exports_1 = require("parcel-plugin-typescript/exports");
const host_1 = require("./host");
const route_1 = require("./route");
const build_loop_1 = require("./build-loop");
class AngularCompiler {
    constructor(project, compileResource) {
        this.transformers = [];
        this.entryFile = null;
        this.config = compiler_cli_1.readConfiguration(project);
        const { options } = this.config;
        this.host = new host_1.AngularCompilerHost(this.config.options, compileResource);
        this.loop = new build_loop_1.BuildLoop(this.host, this.config, this.transformers);
        this.transformers.push(exports_1.PathTransform(options));
    }
    /**
     * Triggers a TypeScript compilation if needed and returns a compiled file
     * @param path a absolute path to a TypeScript source file to compile
     */
    compile(path) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { config, loop } = this;
            if (loop.entryFile === null) {
                // We assume the file file included by the project is the entry
                // It is used to inject the generated SystemJS loader
                // TODO: use Parcel dependencies instead
                loop.entryFile = path;
            }
            yield loop.emit(undefined);
            const { basePath, outDir } = config.options;
            if (basePath && outDir) {
                path = path.replace(basePath, outDir);
            }
            let js = this.host.store.readFile(path.replace(/\.tsx?$/, '.js'));
            const program = loop.getProgram();
            // detect if the file is the main module
            // TODO: use Parcel dependencies
            if (program && path === this.entryFile) {
                js = `${js}\n${route_1.generateRouteLoader(path, program)}`;
            }
            return {
                sources: { js },
                resources: this.loop.getResources(path)
            };
        });
    }
}
exports.AngularCompiler = AngularCompiler;
//# sourceMappingURL=index.js.map