"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const exports_1 = require("parcel-plugin-typescript/exports");
const template_1 = require("../../frontend/loaders/template");
const get_watcher_1 = require("../../utils/get-watcher");
class AngularWorker extends exports_1.Worker {
    constructor(bundler) {
        // We append the socket path to process.env beforce spawning the worker
        exports_1.setSocketPath('angular');
        super(require.resolve('./launcher'));
        this.bundler = bundler;
        this.resources = new Map();
    }
    getFactories(file) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            try {
                const factories = yield this.request('getFactories', undefined);
                const base = file.replace(/\.tsx?/, '');
                return factories.filter(factory => factory.indexOf(base) === 0);
            }
            catch (_a) {
                return [];
            }
        });
    }
    compile(data) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { resources, sources } = yield this.request('compile', data);
            const result = {
                sources,
                resources: {
                    bundled: resources,
                    external: resources
                        .map(resource => template_1.getFileResources(resource))
                        .reduce((a, b) => a.concat(b), [])
                }
            };
            for (const dep of resources.concat(result.resources.external)) {
                const deps = this.resources.get(dep);
                if (!deps) {
                    this.resources.set(dep, [data.file]);
                }
                else {
                    deps.push(data.file);
                }
            }
            return result;
        });
    }
    typeCheck(data) {
        return this.request('typeCheck', data);
    }
    readVirtualFile(file) {
        return this.request('readVirtualFile', file);
    }
    processResource(file) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { package: pkg, options, parser } = this.bundler;
            const source = yield exports_1.readFile(file);
            return template_1.processResource(file, pkg, options, parser, source);
        });
    }
    invalidate(files) {
        return this.request('invalidate', files);
    }
}
tslib_1.__decorate([
    exports_1.HandlerMethod
], AngularWorker.prototype, "compile", null);
tslib_1.__decorate([
    exports_1.HandlerMethod
], AngularWorker.prototype, "typeCheck", null);
tslib_1.__decorate([
    exports_1.HandlerMethod
], AngularWorker.prototype, "readVirtualFile", null);
tslib_1.__decorate([
    exports_1.HandlerMethod
], AngularWorker.prototype, "processResource", null);
tslib_1.__decorate([
    exports_1.HandlerMethod
], AngularWorker.prototype, "invalidate", null);
exports.AngularWorker = AngularWorker;
class AngularServer extends exports_1.Server {
    constructor(bundler) {
        const worker = new AngularWorker(bundler);
        super('angular', worker);
        this.worker = worker;
        this.watch(bundler);
    }
    close() {
        this.worker.kill();
        super.close();
    }
    watch(bundler) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const watcher = yield get_watcher_1.getWatcher(bundler);
            if (!watcher) {
                return;
            }
            const { worker } = this;
            watcher.on('change', (file) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                const deps = worker.resources.get(file);
                const files = [];
                worker.invalidate([file]);
                if (deps) {
                    files.push(...deps);
                }
                files.forEach(depFile => bundler.onChange(depFile));
                const factories = yield Promise.all(files.map(depFile => worker.getFactories(depFile)));
                factories
                    .reduce((a, b) => a.concat(b), [])
                    .forEach(factory => bundler.onChange(factory));
                // TODO: batch this
                worker.invalidate(files);
            }));
        });
    }
}
exports.AngularServer = AngularServer;
//# sourceMappingURL=index.js.map