"use strict";
const tslib_1 = require("tslib");
const exports_1 = require("parcel-plugin-typescript/exports");
const ngc_1 = require("../ngc");
const client_1 = require("./client");
const compilers = new Map();
const services = new Map();
const store = exports_1.FileStore.shared();
function getCompiler(tsConfig) {
    let compiler = compilers.get(tsConfig);
    if (!compiler) {
        compiler = exports_1.loadConfiguration(tsConfig).then(config => new ngc_1.AngularCompiler(config.path, client_1.IPCClient.processResource));
        compilers.set(tsConfig, compiler);
    }
    return compiler;
}
function getService(tsConfig) {
    let service = services.get(tsConfig);
    if (!service) {
        service = exports_1.loadConfiguration(tsConfig).then(config => new exports_1.LanguageService(config));
        services.set(tsConfig, service);
    }
    return service;
}
const handler = {
    compile({ file, tsConfig }) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const compiler = yield getCompiler(tsConfig);
            return compiler.compile(file);
        });
    },
    typeCheck({ file, tsConfig }) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const service = yield getService(tsConfig);
            // TODO: support noEmitOnError
            yield service.check(file, true);
        });
    },
    wait() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield Promise.all(Array
                .from(compilers.values())
                .map(promise => promise.then(compiler => compiler.loop.wait())));
        });
    },
    readVirtualFile(file) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            try {
                yield handler.wait(undefined);
                return store.readFile(file) || null;
            }
            catch (_a) {
                return null;
            }
        });
    },
    invalidate(files) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            for (const file of files) {
                store.invalidate(file);
            }
        });
    },
    getFactories() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield Promise.all(Array
                .from(compilers.values())
                .map(promise => promise.then(compiler => compiler.loop.waitCurrentOrNext())));
            return store
                .getFiles()
                .filter(file => /\.ng(factory)|(style)\.js$/.test(file));
        });
    }
};
module.exports = handler;
//# sourceMappingURL=launcher.js.map