"use strict";
const tslib_1 = require("tslib");
const JSAsset = require("parcel-bundler/lib/assets/JSAsset");
const exports_1 = require("parcel-plugin-typescript/exports");
const resources_1 = require("../../backend/transformers/resources");
const client_1 = require("../../backend/worker/client");
const template_1 = require("../loaders/template");
const collect_dependencies_1 = require("../utils/collect-dependencies");
module.exports = class TSAsset extends JSAsset {
    constructor(name, pkg, options) {
        super(name, pkg, options);
        this.resources = {
            bundled: [],
            external: []
        };
        this.templatePreProcessor = {
            findExpr: /_PRAGMA_PARCEL_TYPESCRIPT_PLUGIN_PREPROCESS_TEMPLATE\(([^\)]*)\)/g,
            replaceExpr: /._PRAGMA_PARCEL_TYPESCRIPT_PLUGIN_PREPROCESS_TEMPLATE\(([^\)]*)\)./g,
            transform: path => template_1.processResource(path, this.package, this.options, this.options.parser)
        };
        this.stylePreProcessor = {
            findExpr: /_PRAGMA_PARCEL_TYPESCRIPT_PLUGIN_PREPROCESS_STYLE\(([^\)]*)\)/g,
            replaceExpr: /._PRAGMA_PARCEL_TYPESCRIPT_PLUGIN_PREPROCESS_STYLE\(([^\)]*)\)./g,
            // TODO: preprocess CSS
            transform: path => template_1.processResource(path, this.package, this.options, this.options.parser)
        };
        this.config = exports_1.loadConfiguration(name);
        this.transpiler = this.config.then(config => new exports_1.Transpiler(config, [resources_1.replaceResources(() => true)]));
    }
    parse(code) {
        const _super = name => super[name];
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { path: tsConfig } = yield this.config;
            client_1.IPCClient.typeCheck({ tsConfig, file: this.name });
            const transpiler = yield this.transpiler;
            const { sources } = transpiler.transpile(code, this.name);
            this.resources.bundled.splice(0);
            this.resources.external.splice(0);
            this.contents = yield this.preProcessResources(sources.js, this.templatePreProcessor, this.stylePreProcessor);
            // Parse result as ast format through babylon
            return _super("parse").call(this, this.contents);
        });
    }
    collectDependencies() {
        super.collectDependencies();
        collect_dependencies_1.collectDependencies(this, this.resources);
    }
    preProcessResources(code, ...preProcessors) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const results = preProcessors
                .map(preProcessor => ({
                matches: code.match(preProcessor.findExpr) || [],
                preProcessor
            }));
            const resources = {};
            yield Promise.all(results
                .map(({ matches, preProcessor }) => matches.map(match => {
                const found = preProcessor.findExpr.exec(match);
                if (found) {
                    return found.pop();
                }
            })
                .filter(match => !!match)
                .map((match) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                const path = new Buffer(match, 'base64').toString('utf-8');
                resources[match] = yield preProcessor.transform(path);
                this.resources.external.push(...template_1.getFileResources(path));
                this.resources.bundled.push(path);
            })))
                .reduce((a, b) => a.concat(b), []));
            results.forEach(({ preProcessor }) => code = code.replace(preProcessor.replaceExpr, (_, path) => JSON.stringify(resources[path])));
            return code;
        });
    }
};
//# sourceMappingURL=jit.js.map