"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const path_1 = require("path");
const template_1 = require("../assets/template");
const cache = new Map();
const fileResources = new Map();
function processResource(file, pkg, options, parser, code) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const cached = cache.get(file);
        if (cached && cached.original === code) {
            return cached.generated;
        }
        const asset = /\.html/.test(file)
            ? new template_1.TemplateAsset(file, pkg, options)
            : parser.getAsset(file, pkg, options);
        if (code) {
            asset.contents = code;
        }
        const processed = yield asset.process();
        const generated = processed[asset.type];
        cache.set(file, {
            original: asset.contents,
            generated
        });
        fileResources.set(file, Array
            .from(asset.dependencies.keys())
            .filter(key => !/^_css_loader/.test(key))
            .map(k => path_1.resolve(path_1.dirname(file), k)));
        return generated;
    });
}
exports.processResource = processResource;
function getFileResources(file) {
    const resource = fileResources.get(file);
    if (!resource) {
        throw new Error(`Cannot find resources for file ${file}`);
    }
    return resource;
}
exports.getFileResources = getFileResources;
//# sourceMappingURL=template.js.map