"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
/**
 * When generating files on the fly on AOT mode we need to
 * override Parcel cache and resolver for it to work properly
 */
function injectAngularSupport(bundler) {
    const { cache, resolver } = bundler;
    if (cache) {
        const Cache = cache.constructor;
        const CacheWrite = Cache.prototype.write;
        const CacheRead = Cache.prototype.read;
        cache.write = function (file, data) {
            if (/\.ng(factory)|(style)\.js$/.test(file) || /\.ts$/.test(file)) {
                return Promise.resolve();
            }
            return CacheWrite.call(this, file, data);
        };
        cache.read = function (file) {
            if (/\.ng(factory)|(style)\.js$/.test(file)) {
                return Promise.resolve(null);
            }
            return CacheRead.call(this, file);
        };
    }
    const Resolver = resolver.constructor;
    const ResolveInternal = Resolver.prototype.resolveInternal;
    resolver.resolveInternal = function (path, parent, ...args) {
        if (/\.ngfactory$/.test(path) || /\.ngstyle$/.test(path)) {
            return path_1.resolve(path_1.dirname(parent), `${path}.js`);
        }
        if (/\.ngfactory\.js$/.test(path) || /\.ngstyle\.js$/.test(path)) {
            return path_1.resolve(path_1.dirname(parent), path);
        }
        return ResolveInternal.call(this, path, parent, ...args);
    };
}
exports.injectAngularSupport = injectAngularSupport;
//# sourceMappingURL=patch.js.map