"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
class AsyncLoop {
    constructor(work) {
        this.work = work;
        this.running = false;
        this.queue = [];
        this.watchers = [];
    }
    emit(data) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            this.queue.push(data);
            if (this.running) {
                return this.wait();
            }
            this.running = true;
            try {
                yield this.work(this.queue.splice(0));
            }
            finally {
                this.running = false;
                while (this.watchers.length > 0) {
                    this.watchers.pop()();
                }
            }
        });
    }
    wait() {
        if (!this.running) {
            return Promise.resolve();
        }
        return this.waitCurrentOrNext();
    }
    waitCurrentOrNext() {
        return new Promise(resolve => this.watchers.push(resolve));
    }
}
exports.AsyncLoop = AsyncLoop;
//# sourceMappingURL=async-loop.js.map